"use strict";

var Helper = require('../Helper');

require("../mocha-config.js");

var AliasOverlay = require('../../../main/js/map/overlay/AliasOverlay');
var Comment = require('../../../main/js/map/data/Comment');
var ControlType = require('../../../main/js/map/ControlType');
var CustomMap = require('../../../main/js/map/CustomMap');
var IdentifiedElement = require('../../../main/js/map/data/IdentifiedElement');

var ReactionOverlay = require('../../../main/js/map/overlay/ReactionOverlay');

var logger = require('./../logger');

var chai = require('chai');
var assert = chai.assert;

describe('CustomMap', function() {

  var helper;
  before(function() {
    helper = new Helper();
  });

  it("contructor", function() {
    var options = helper.createCustomMapOptions();
    var map = new CustomMap(options);
    assert.ok(map);
  });

  it("contructor", function() {
    var options = helper.createCustomMapOptions();
    var map = new CustomMap(options);
    assert.ok(map);
  });

  it("contructor with submaps", function() {
    var options = helper.createCustomMapOptions();

    options.getProject().getModel().addSubmodel(helper.createModel());

    var map = new CustomMap(options);
    assert.ok(map);
  });

  it("getSubmodelById", function() {
    var map = helper.createCustomMap();
    assert.ok(map.getSubmodelById(map.getId()));
  });

  it("getSubmodelById (invalid)", function() {
    var map = helper.createCustomMap();
    assert.equal(map.getSubmodelById(-1), null);
  });

  it("getSubmodelById (string id)", function() {
    var map = helper.createCustomMap();
    assert.ok(map.getSubmodelById(map.getId() + ""));
  });

  it("openLayoutById (not existing)", function() {
    var map = helper.createCustomMap();
    try {
      map.openLayoutById(-1);
      assert.ok(false);
    } catch (exception) {
      assert.ok(exception.message.indexOf("You have no privileges") >= 0);
    }
  });

  it("openLayout", function() {
    var options = helper.createCustomMapOptions();
    var model = options.getProject().getModel();
    var layout = model.getLayouts()[0];
    var map = new CustomMap(options);
    map.openLayoutById(layout.getId());
    assert.equal(logger.getErrors().length, 0);
  });

  it("openLayout (string id)", function() {
    var options = helper.createCustomMapOptions();
    var model = options.getProject().getModel();
    var layout = model.getLayouts()[0];
    var map = new CustomMap(options);
    map.openLayoutById(layout.getId() + "");
    assert.equal(logger.getErrors().length, 0);
  });

  it("addSelectedLayout", function() {
    var map = helper.createCustomMap();
    var layout = helper.createLayout();
    layout.setInitialized(true);
    map.getModel().addLayout(layout);

    return map.addSelectedLayout(layout.getId()).then(function() {
      var vLayouts = ServerConnector.getSessionData(map.getProject()).getVisibleOverlays();
      assert.equal(1, vLayouts.length);
      assert.equal(layout.getId(), vLayouts[0]);
    });
  });

  it("removeSelectedLayout", function() {
    var map = helper.createCustomMap();
    var layout = helper.createLayout();
    layout.setInitialized(true);
    map.getModel().addLayout(layout);

    return map.addSelectedLayout(layout.getId()).then(function() {
      map.removeSelectedLayout(layout.getId());
      var vLayouts = ServerConnector.getSessionData(map.getProject()).getVisibleOverlays();
      assert.equal(0, vLayouts.length);
    });
  });

  it("addSelectedLayout with data download", function() {
    var map = helper.createCustomMap();
    var layout = helper.createLayout();
    layout.setId(102);
    map.getModel().addLayout(layout);

    return map.addSelectedLayout(layout.getId()).then(function() {
      var vLayouts = ServerConnector.getSessionData(map.getProject()).getVisibleOverlays();
      assert.equal(1, vLayouts.length);
      assert.equal(layout.getId(), vLayouts[0]);

      var overlays = map.getSelectedLayoutOverlays()[layout.getId()];
      assert.equal(overlays.length, 1);
      assert.ok(overlays[0] instanceof AliasOverlay);
    });
  });

  it("_showSelectedLayout", function() {
    var map = helper.createCustomMap();
    var alias = helper.createAlias();
    alias.setModelId(map.getId());
    map.getModel().addAlias(alias);

    var reaction = helper.createReaction();
    map.getModel().addReaction(reaction);

    var layout = helper.createLayout();
    layout.setInitialized(true);
    var layoutAlias = helper.createLayoutAlias();
    layoutAlias.setId(alias.getId());
    layout.addAlias(layoutAlias);

    var layoutReaction = helper.createLayoutReaction();
    layoutReaction.setId(reaction.getId());
    layout.addReaction(layoutReaction);

    map.getModel().addLayout(layout);

    return map.addSelectedLayout(layout.getId()).then(function() {
      return map._showSelectedLayout(layout.getId(), 0, 1);
    }).then(function() {

      assert.equal(2, map.selectedLayoutOverlays[layout.getId()].length);
      assert.ok(map.selectedLayoutOverlays[layout.getId()][0] instanceof AliasOverlay);
      assert.ok(map.selectedLayoutOverlays[layout.getId()][1] instanceof ReactionOverlay);

      // now hide the layout
      map._hideSelectedLayout(layout.getId());
      assert.ok(map.selectedLayoutOverlays[layout.getId()]);
      assert.equal(0, map.selectedLayoutOverlays[layout.getId()].length);
    });

  });
  it("openInfoWindowForAlias", function() {
    var map = helper.createCustomMap();
    var layout = helper.createLayout();
    var alias = helper.createAlias();
    alias.setModelId(map.getId());
    var layoutAlias = helper.createLayoutAlias();
    layoutAlias.setId(alias.getId());

    layout.addAlias(layoutAlias);

    // create layout
    map.getModel().addLayout(layout);
    map.getModel().addAlias(alias);

    return map.openInfoWindowForAlias(alias.getId(), map.getId(), function() {
      assert.ok(map.getAliasInfoWindowById(alias.getId()));
      assert.ok(map.getAliasInfoWindowById(alias.getId()).isOpened());
    });
  });

  it("openInfoWindowForAlias for incomplete alias", function() {
    var map = helper.createCustomMap();
    var layout = helper.createLayout();
    var alias = helper.createAlias(map);
    alias.setId(30001);
    alias.setIsComplete(false);
    var layoutAlias = helper.createLayoutAlias(alias);

    layout.addAlias(layoutAlias);

    // create layout
    map.getModel().addLayout(layout);
    map.getModel().addAlias(alias);

    return map.openInfoWindowForAlias(alias.getId(), map.getId(), function() {
      assert.ok(map.getAliasInfoWindowById(alias.getId()));
      assert.ok(map.getAliasInfoWindowById(alias.getId()).isOpened());
    });
  });

  it("getSearchAutocomplete", function() {
    var map = helper.createCustomMap();

    var t = map.getSearchAutocomplete("s");
    assert.ok(t);
    assert.equal(t.length, 0);
  });

  it("getSearchAutocomplete 2", function() {
    var map = helper.createCustomMap();

    return map.refreshSearchAutocomplete("s").then(function() {
      var t = map.getSearchAutocomplete("s");
      assert.ok(t);
      assert.ok(t.length > 0);
    });
  });

  it("renderOverlayCollection for alias", function() {
    var map = helper.createCustomMap();
    var reaction = helper.createReaction();
    var alias = helper.createAlias();
    map.getModel().addAlias(alias);
    map.getModel().addReaction(reaction);

    var oc = helper.createDbOverlay(map);

    oc.getIdentifiedElements = function() {
      return new Promise(function(resolve) {
        resolve([ new IdentifiedElement({
          objectId : alias.getId(),
          icon : "empty.png",
          modelId : map.getId(),
          type : "Alias"
        }) ]);
      });
    };

    return map.renderOverlayCollection({
      overlayCollection : oc
    }).then(function() {
      var aliasMarkerCount = 0;
      for ( var id in oc.aliasMarkers) {
        if (oc.aliasMarkers.hasOwnProperty(id)) {
          aliasMarkerCount++;
        }
      }
      assert.equal(1, aliasMarkerCount);
    });
  });

  it("renderOverlayCollection for set of aliases", function() {
    var map = helper.createCustomMap();

    var oc = helper.createDbOverlay(map);

    var identifiedElements = [];
    for (var i = 0; i < 3; i++) {
      var alias = helper.createAlias(map);
      map.getModel().addAlias(alias);
      identifiedElements.push(helper.createIdentifiedElement(alias));
    }

    oc.getIdentifiedElements = function() {
      return new Promise(function(resolve) {
        resolve(identifiedElements);
      });
    };

    return map.renderOverlayCollection({
      overlayCollection : oc
    }).then(function() {
      var aliasMarkerCount = 0;
      for ( var id in oc.aliasMarkers) {
        if (oc.aliasMarkers.hasOwnProperty(id)) {
          aliasMarkerCount++;
        }
      }

      assert.equal(aliasMarkerCount, 3);
    });
  });

  it("renderOverlayCollection for point", function() {
    var map = helper.createCustomMap();

    var oc = helper.createDbOverlay(map);

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };

    oc.getIdentifiedElements = function() {
      return new Promise(function(resolve) {
        resolve([ new IdentifiedElement(javaObj) ]);
      });
    };

    return map.renderOverlayCollection({
      overlayCollection : oc
    }).then(function() {
      var markerCount = 0;
      for ( var id in oc.pointMarkers) {
        if (oc.pointMarkers.hasOwnProperty(id)) {
          markerCount++;
        }
      }
      assert.equal(1, markerCount);
    });
  });

  it("renderOverlayCollection for reaction", function() {
    var map = helper.createCustomMap();
    var reaction = helper.createReaction();
    map.getModel().addReaction(reaction);

    var oc = helper.createDbOverlay(map);

    var javaObj = {
      objectId : reaction.getId(),
      modelId : map.getId(),
      type : "Reaction",
    };

    oc.getIdentifiedElements = function() {
      return new Promise(function(resolve) {
        resolve([ new IdentifiedElement(javaObj) ]);
      });
    };

    return map.renderOverlayCollection({
      overlayCollection : oc
    }).then(function() {
      var markerCount = 0;
      for ( var id in oc.reactionMarkers) {
        if (oc.reactionMarkers.hasOwnProperty(id)) {
          markerCount++;
        }
      }
      assert.equal(1, markerCount);
    });
  });

  it("clearData", function() {
    var map = helper.createCustomMap();

    var oc = helper.createDbOverlay(map);

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };

    oc.getIdentifiedElements = function() {
      return new Promise(function(resolve) {
        resolve([ new IdentifiedElement(javaObj) ]);
      });
    };

    map.renderOverlayCollection({
      overlayCollection : oc
    });

    map.clearData();

    var markerCount = 0;
    for ( var id in oc.pointMarkers) {
      if (oc.pointMarkers.hasOwnProperty(id)) {
        markerCount++;
      }
    }

    assert.equal(0, markerCount);

  });

  it("getInfoWindowForIdentifiedElement ( reaction)", function() {
    var map = helper.createCustomMap();
    var reaction = helper.createReaction(map);
    map.getModel().addReaction(reaction);

    var ie = helper.createIdentifiedElement(reaction);

    var infoWindow = map.getInfoWindowForIdentifiedElement(ie);

    assert.equal(null, infoWindow);
    return map._openInfoWindowForReaction(reaction.getId()).then(function() {
      infoWindow = map.getInfoWindowForIdentifiedElement(ie);

      assert.ok(infoWindow);
    });

  });

  it("right click on map", function() {
    var map = helper.createCustomMap();

    var mev = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getActiveSubmapId());
    google.maps.event.trigger(map.getGoogleMap(), 'rightclick', mev);
    assert.equal(map.getId(), map.getActiveSubmapId());

  });

  it("left click on map", function() {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);

    var searchOverlay = helper.createSearchDbOverlay(map);

    var mev = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getActiveSubmapId());
    return google.maps.event.trigger(map.getGoogleMap(), 'click', mev).then(function() {
      assert.equal(map.getId(), map.getActiveSubmapId());
      assert.ok(searchOverlay.aliasMarkers[329173]);
    });
  });

  it("left click on reaction", function() {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);

    var searchOverlay = helper.createSearchDbOverlay(map);

    var mev = {
      stop : null,
      latLng : new google.maps.LatLng(42.0, -90.0)
    };

    assert.notOk(map.getActiveSubmapId());
    return google.maps.event.trigger(map.getGoogleMap(), 'click', mev).then(function() {
      assert.equal(map.getId(), map.getActiveSubmapId());
      assert.ok(searchOverlay.reactionMarkers[153515]);
      assert.ok(searchOverlay.reactionMarkers[153515].isShown());
      assert.ok(searchOverlay.aliasMarkers[329162]);
    });
  });

  it("create polygon through drawingManager", function() {
    var map = helper.createCustomMap();

    map.turnOnDrawing();

    var triangleCoords = [ new google.maps.LatLng(25.774, -80.190), new google.maps.LatLng(18.466, -66.118),
        new google.maps.LatLng(32.321, -64.757), new google.maps.LatLng(25.774, -80.190), ];

    // Construct the polygon.
    var bermudaTriangle = new google.maps.Polygon({
      paths : triangleCoords,
      strokeColor : '#FF0000',
      strokeOpacity : 0.8,
      strokeWeight : 2,
      fillColor : '#FF0000',
      fillOpacity : 0.35
    });
    bermudaTriangle.setMap(map.getGoogleMap());

    var eventParam = {
      type : google.maps.drawing.OverlayType.POLYGON,
      overlay : bermudaTriangle,
    };

    // check creation complete behaviour
    assert.ok(map.isDrawingOn());
    assert.equal(google.maps.drawing.OverlayType.POLYGON, map._drawingManager.getDrawingMode());
    google.maps.event.trigger(map._drawingManager, 'overlaycomplete', eventParam);
    assert.ok(map.isDrawingOn());
    assert.equal(null, map._drawingManager.getDrawingMode());

    // and now check how it behaves when we wiught click on the polygon
    var clickEventParam = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getSelectedPolygon());

    assert.notOk(map.getActiveSubmapId());
    assert.notOk(map.getSelectedArea());
    google.maps.event.trigger(bermudaTriangle, 'rightclick', clickEventParam);
    assert.ok(map.getSelectedArea());
    assert.equal(map.getId(), map.getActiveSubmapId());

    assert.ok(map.getSelectedPolygon());

  });

  it("getAliasVisibleLayoutsData", function() {
    var mockObject = helper.createCustomMap();

    var alias = helper.createAlias();
    mockObject.getModel().addAlias(alias);

    var layout = helper.createLayout();
    layout.setInitialized(true);
    mockObject.getModel().addLayout(layout);

    return mockObject.addSelectedLayout(layout.getId()).then(function() {
      return mockObject.getAliasVisibleLayoutsData(alias.getId());
    }).then(function(layoutAliases) {
      assert.equal(layoutAliases.length, 1);
    });
  });

  it("changed coords in map", function() {
    var map = helper.createCustomMap();
    var oldCenter = map.getGoogleMap().getCenter();
    var newCenter = new google.maps.LatLng(3, 87);
    map.getGoogleMap().setCenter(newCenter);
    google.maps.event.trigger(map.getGoogleMap(), "center_changed");

    var center = ServerConnector.getSessionData(map.getProject()).getCenter(map.getModel());
    assert.ok(center !== oldCenter);
    assert.ok(center instanceof google.maps.Point);
  });

  it("refreshComments", function() {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    var commentsOverlay = helper.createCommentDbOverlay(map);

    ServerConnector.getSessionData().setShowComments(true);
    return map.refreshComments().then(function() {
      assert.ok(commentsOverlay.pointMarkers['(241.01,372.35)']);
      assert.ok(commentsOverlay.pointMarkers['(643.96,144.09)']);
      assert.notOk(commentsOverlay.pointMarkers['unkId']);
    });
  });

  it("hide comments", function() {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    var commentsOverlay = helper.createCommentDbOverlay(map);

    ServerConnector.getSessionData().setShowComments(true);
    return map.refreshComments().then(function() {
      ServerConnector.getSessionData().setShowComments(false);
      return map.refreshComments();
    }).then(function() {
      assert.notOk(commentsOverlay.pointMarkers['(241.01, 372.35)']);
    });
  });

  it("openCommentDialog", function() {
    var map = helper.createCustomMap();
    map.getModel().setId(102);
    map.setActiveSubmapId(102);
    map.setActiveSubmapClickCoordinates(new google.maps.Point(2, 12));
    return map.openCommentDialog().then(function() {
      var types = map.getCommentDialog().getTypes();
      assert.equal(types.length, 3);
      var selected = map.getCommentDialog().getSelectedType();
      assert.ok(selected === "<General>");

      map.getCommentDialog().setSelectedType(1);
      selected = map.getCommentDialog().getSelectedType();
      assert.notOk(selected === "<General>");

      map.getCommentDialog().setSelectedType(2);
      selected = map.getCommentDialog().getSelectedType();
      assert.notOk(selected === "<General>");
    });
  });

  it("addComment", function() {
    var map = helper.createCustomMap();
    map.getModel().setId(102);
    map.setActiveSubmapId(102);
    map.setActiveSubmapClickCoordinates(new google.maps.Point(2, 12));
    return map.openCommentDialog().then(function() {
      return map.getCommentDialog().addComment();
    });
  });

  it("retrieveOverlayDetailDataForElement for comment", function() {
    var map = helper.createCustomMap();
    helper.createCommentDbOverlay(map);

    var alias = helper.createAlias(map);
    alias.setId(329158);
    map.getModel().addAlias(alias);

    var ie = new IdentifiedElement(alias);

    return map.retrieveOverlayDetailDataForElement(ie, {
      comment : true
    }).then(function(details) {
      assert.ok(details);
      assert.equal(details.length, 1);
      assert.equal(details[0].length, 1);
      assert.ok(details[0][0] instanceof Comment);
    });
  });

  it("getOverlayDataForIdentifiedElement", function() {
    var map = helper.createCustomMap();
    var commentOverlay = helper.createCommentDbOverlay(map);

    var alias = helper.createAlias(map);
    alias.setId(329158);
    map.getModel().addAlias(alias);

    var ie = new IdentifiedElement(alias);

    return map.getOverlayDataForIdentifiedElement(ie, {
      comment : true
    }).then(function(details) {
      assert.equal(details.length, 1);

      var overlayData = details[0];
      assert.equal(overlayData.overlay, commentOverlay);
      assert.ok(overlayData.data);
      assert.equal(overlayData.data.length, 1);
      assert.ok(overlayData.data[0] instanceof Comment);
    });
  });

  it("openSubmodel", function() {
    var options = helper.createCustomMapOptions();

    var submodel = helper.createModel();
    options.getProject().getModel().addSubmodel(submodel);
    
    var map = new CustomMap(options);
    
    map.openSubmodel(submodel.getId());
  });

});
