"use strict";

var Helper = require('./helper');

require("./mocha-config.js");

var minerva = require('../../main/js/minerva');
var Project = require('../../main/js/map/data/Project');

var chai = require('chai');
var assert = chai.assert;
var logger = require('./logger');

describe('minerva global', function() {

  var helper;
  before(function() {
    helper = new Helper();
  });

  beforeEach(function() {
    global.leftPanelTab = helper.createLeftPanelTab();
    document.body.appendChild(global.leftPanelTab);

    global.menuDiv = helper.createMenuDiv();
    document.body.appendChild(global.menuDiv);
    
    global.dialogsDiv = helper.createDialogsDiv();
    document.body.appendChild(global.dialogsDiv);

    global.scriptDiv = document.createElement("script");
    document.body.appendChild(global.scriptDiv);
});

  afterEach(function() {
    document.body.removeChild(global.leftPanelTab);
    document.body.removeChild(global.menuDiv);
    document.body.removeChild(global.dialogsDiv);
    document.body.removeChild(global.scriptDiv);
  });

  it('create', function() {
    var options = helper.createOptions();

    return minerva.create(options).then(function(result) {
      assert.ok(result);
      assert.equal(logger.getWarnings().length, 1);
    });
  });

  it('create with overview', function() {
    return ServerConnector.readFile("testFiles/projectWithImages.json").then(function(fileContent) {
      var project = new Project(JSON.parse(fileContent));
      var options = helper.createOptions(project);

      return minerva.create(options);
    }).then(function(result) {
      assert.ok(result);
      assert.equal(logger.getWarnings().length, 1);
      var showOverviewButton = document.getElementsByName("showOverviewButton")[0];
      showOverviewButton.onclick();
    });
  });

  it("showComments", function() {
    var options = helper.createCustomMapOptions();
    return minerva.create(options).then(function() {
      document.getElementsByName("commentCheckbox")[0].click();
      assert.ok(ServerConnector.getSessionData(options.getProject()).getShowComments());
    });
  });

  it("contructor with GET zoom param", function() {
    var options = helper.createCustomMapOptions();
    GuiConnector.getParams["zoom"] = "5";
    return minerva.create(options).then(
        function() {
          assert.equal(ServerConnector.getSessionData(options.getProject()).getZoomLevel(
              options.getProject().getModel()), 5);
        });
  });

  it("contructor with GET coord param", function() {
    var options = helper.createCustomMapOptions();
    GuiConnector.getParams["x"] = "5";
    GuiConnector.getParams["y"] = "6";
    return minerva.create(options).then(function() {
      var center = ServerConnector.getSessionData(options.getProject()).getCenter(options.getProject().getModel());
      assert.ok(center instanceof google.maps.Point);
      assert.equal(center.x, 5);
      assert.equal(center.y, 6);
    });
  });

  it('create with layout', function() {
    var project = helper.createProject();

    var layout = helper.createLayout();
    layout.setInputDataAvailable(false);
    var layoutName = layout.getName();
    var layoutId = layout.getId();

    project.getModel().addLayout(layout);

    GuiConnector.getParams["layout"] = layoutName;

    var options = helper.createOptions(project);

    return minerva.create(options).then(function(result) {
      assert.ok(result);
      // input file is not available so it's the background
      assert.equal(result.getSelectedLayouts().length, 0);
      assert.equal(ServerConnector.getSessionData(project).getSelectedBackgroundOverlay(), "cv" + layoutId);
      assert.equal(logger.getWarnings().length, 1);
    });
  });

  it('create with layout from session data', function() {
    var project = helper.createProject();

    var layout = helper.createLayout();
    layout.setInputDataAvailable(false);
    var layoutName = layout.getName();

    project.getModel().addLayout(layout);

    ServerConnector.getSessionData(project).setSelectedBackgroundOverlay(layoutName);

    var options = helper.createOptions(project);

    return minerva.create(options);
  });

  it('create with layout 2', function() {
    var project = helper.createProject();

    var layout = helper.createLayout();
    layout.setInputDataAvailable(true);
    // disable upload of the data from server
    layout.setInitialized(true);
    var layoutName = layout.getName();

    project.getModel().addLayout(layout);

    GuiConnector.getParams["layout"] = layoutName;

    var options = helper.createOptions(project);

    return minerva.create(options).then(function(result) {
      assert.ok(result);
      // input file is available so it's not the background file but overlay
      assert.equal(result.getSelectedLayouts().length, 1);
      assert.equal(logger.getWarnings().length, 1);
    });
  });

  it('create with comment overlay', function() {
    var project = helper.createProject();
    project.getModel().setId(15781);

    var options = helper.createOptions(project);
    options.dataCollections.push({
      name : "comment"
    });

    return minerva.create(options).then(function(result) {
      assert.ok(result);
    });
  });

  it('create with search overlay', function() {
    var project = helper.createProject();
    project.getModel().setId(15781);

    var options = helper.createOptions(project);
    options.dataCollections.push({
      name : "search"
    });

    return minerva.create(options).then(function(result) {
      assert.ok(result);
    });
  });

  it('create with drug overlay', function() {
    var project = helper.createProject();
    project.getModel().setId(15781);

    var options = helper.createOptions(project);
    options.dataCollections.push({
      name : "drug"
    });

    return minerva.create(options).then(function(result) {
      assert.ok(result);
    });
  });

  it('create with chemical overlay', function() {
    var project = helper.createProject();
    project.getModel().setId(15781);

    var options = helper.createOptions(project);
    options.dataCollections.push({
      name : "chemical"
    });

    return minerva.create(options).then(function(result) {
      assert.ok(result);
    });
  });

  it('create with mirna overlay', function() {
    var project = helper.createProject();
    project.getModel().setId(15781);

    var options = helper.createOptions(project);
    options.dataCollections.push({
      name : "mirna"
    });

    return minerva.create(options).then(function(result) {
      assert.ok(result);
    });
  });

  it('create with search overlay and GET search param', function() {
    GuiConnector.getParams["search"] = "s1";

    var project = helper.createProject();
    project.getModel().setId(15781);

    var options = helper.createOptions(project);
    options.dataCollections.push({
      name : "search"
    });

    return minerva.create(options).then(function(result) {
      return result.getOverlayByName("search").getIdentifiedElements();
    }).then(function(elements) {
      assert.ok(elements.length > 0);
    });
  });

  it("contructor with touch enabled", function() {
    var options = helper.createCustomMapOptions();
    options.setCustomTouchInterface(true);
    return minerva.create(options).then(function(result) {
      assert.ok(result);
    });
  });

});
